==> adeptutils-0.0.2.tar.gz: lib/src/extendables/adept_stateful_widget.dart <==
import 'package:flutter/material.dart';
import 'dart:async';
import 'package:adeptutils/src/interfaces/iwidget.dart';
import 'package:adeptutils/src/extendables/adept_state.dart';
import 'package:adeptutils/src/interfaces/ibase.dart';
import 'package:adeptutils/src/exception_utils.dart';

abstract class AdeptStatefulWidget<S extends AdeptState, T> extends StatefulWidget implements IWidget<T>{
  S state;
  IBase _base;
  IBase get base => _base;
  bool _pressBackAgain = false;
  String _pressBackAgainMessage;
  BuildContext _context;
  BuildContext get context => _context;


  AdeptStatefulWidget({IBase base, bool pressBackAgain = false, String pressBackAgainMessage = "Press back again to exit"}) :
        _base = base,
        _pressBackAgain = pressBackAgain,
        _pressBackAgainMessage = pressBackAgainMessage
  ;

  void handleException(dynamic e,
      {
        BuildContext context, String title = "Error",
        String btnContinueText = "CONTINUE",
        String btnExitText = "EXIT"
      }) {
    if(context == null) context = this.context;
    ExceptionUtils.handleException(e, context, title: title,
        btnContinueText: btnContinueText, btnExitText: btnExitText);
  }

  @mustCallSuper
  Widget build(BuildContext context) {
    _context = context;
    return null;
  }

  @override
  void showLoading({String message, bool showProgressIndeterminate = true, Widget progressIndicator}) {
    if(state != null)
      state.showLoading(
        message: message,
        showProgressIndeterminate: showProgressIndeterminate,
        progressIndicator: progressIndicator
      );
  }

  @override
  void hideLoading({int delayMillis}) {
    if(state != null) state.hideLoading(delayMillis: delayMillis);
  }

  bool get isLoading => state != null ? state.isLoading : false;

  @override
  void extras() {
    if(state != null) state.extras();
  }

  @override
  void init() {
    if(state != null) state.init();
  }

  @override
  void check() {
    if(state != null) state.check();
  }

  @override
  void listeners() {
    if(state != null) state.listeners();
  }

  @override
  bool validate() {
    if(state != null) state.validate();
  }

  @override
  int getSelectedDrawerItemId() {

  }

  @override
  bool isPressBackAgain() {
    if(state != null) return state.isPressBackAgain();
    return false;
  }

  @override
  void pressBackAgain(bool pressBackAgain, {String pressBackAgainMessage = "Press back again to exit"}) {
    if(state != null) state.pressBackAgain(pressBackAgain, pressBackAgainMessage: pressBackAgainMessage);
  }

  @override
  void onBuildCompleted() {
    if(state != null) state.onBuildCompleted();
  }

  /**
   * onBackPressed returns false to stop propagating the event, true in other cases
   */
  @override
  Future<bool> onBackPressed() {
    if(state != null) return state.onBackPressed(); else
      return Future(() => true);
  }

  @override
  Future<T> push<T>({BuildContext context, Route<T> route, Widget widget}) {
    if(context == null) context = _context;
    if(route != null) return Navigator.push(context, route); else
    if(widget != null) return Navigator.push(context, MaterialPageRoute(builder: (context) => widget));
  }


  @override
  Future<T> pushReplacement<T, TO>({BuildContext context, Route<T> newRoute, Widget newWidget, TO result}) {
    if(context == null) context = _context;
    if(newRoute != null) return Navigator.pushReplacement(context, newRoute); else
    if(newWidget != null) return Navigator.pushReplacement(context, MaterialPageRoute(builder: (context) => newWidget));
  }

  @override
  void onClick({Widget w, id}) {
    if(state != null) state.onClick(w: w, id: id);
  }

  @override
  void onLongClick({Widget w, id}) {
    if(state != null) state.onLongClick(w: w, id: id);
  }

  @override
  void onItemLongClick({Widget w, T item, int position, id}) {
    if(state != null) state.onItemLongClick(w: w, item: item, position: position, id: id);
  }

  @override
  void onItemClick({Widget w, T item, int position, id}) {
    if(state != null) state.onItemClick(w: w, item: item, position: position, id: id);
  }

  @override
  void onCreated() {

  }

  @override
  void onResumed() {

  }

  @override
  void onInactive() {

  }

  @override
  void onPaused() {

  }

  @override
  void onSuspending() {

  }

  @override
  void onStopped() {

  }

  @override
  void onDestroy() {

  }

}

==> adeptutils-1.0.1.tar.gz: lib/src/extendables/adept_stateful_widget.dart <==
import 'package:flutter/material.dart';
import 'dart:async';
import 'package:adeptutils/src/interfaces/iwidget.dart';
import 'package:adeptutils/src/extendables/adept_state.dart';
import 'package:adeptutils/src/interfaces/ibase.dart';
import 'package:adeptutils/src/exception_utils.dart';

abstract class AdeptStatefulWidget<S extends AdeptState, T> extends StatefulWidget implements IWidget<T>{
  S state;
  IBase _base;
  IBase get base => _base;
  bool _pressBackAgain = false;
  String _pressBackAgainMessage;
  BuildContext _context;
  BuildContext get context => _context;


  AdeptStatefulWidget({IBase base, bool pressBackAgain = false, String pressBackAgainMessage = "Press back again to exit"}) :
        _base = base,
        _pressBackAgain = pressBackAgain,
        _pressBackAgainMessage = pressBackAgainMessage
  ;

  void handleException(dynamic e,
      {
        BuildContext context, String title = "Error",
        String btnContinueText = "CONTINUE",
        String btnExitText = "EXIT"
      }) {
    if(context == null) context = this.context;
    ExceptionUtils.handleException(e, context, title: title,
        btnContinueText: btnContinueText, btnExitText: btnExitText);
  }

  @mustCallSuper
  Widget build(BuildContext context) {
    _context = context;
    return null;
  }

  @override
  void showLoading({String message, bool showProgressIndeterminate = true, Widget progressIndicator}) {
    if(state != null)
      state.showLoading(
        message: message,
        showProgressIndeterminate: showProgressIndeterminate,
        progressIndicator: progressIndicator
      );
  }

  @override
  void hideLoading({int delayMillis}) {
    if(state != null) state.hideLoading(delayMillis: delayMillis);
  }

  bool get isLoading => state != null ? state.isLoading : false;

  @override
  void extras() {
    if(state != null) state.extras();
  }

  @override
  void init() {
    if(state != null) state.init();
  }

  @override
  void check() {
    if(state != null) state.check();
  }

  @override
  void listeners() {
    if(state != null) state.listeners();
  }

  @override
  bool validate() => state != null ? state.validate() : true;

  @override
  int getSelectedDrawerItemId() => state?.getSelectedDrawerItemId();

  @override
  bool isPressBackAgain() {
    if(state != null) return state.isPressBackAgain();
    return false;
  }

  @override
  void pressBackAgain(bool pressBackAgain, {String pressBackAgainMessage = "Press back again to exit"}) {
    if(state != null) state.pressBackAgain(pressBackAgain, pressBackAgainMessage: pressBackAgainMessage);
  }

  @override
  void onBuildCompleted() {
    if(state != null) state.onBuildCompleted();
  }

  /// onBackPressed returns false to stop propagating the event, true in other cases
  @override
  Future<bool> onBackPressed() {
    if(state != null) return state.onBackPressed(); else
      return Future(() => true);
  }

  @override
  Future<T> push<T>({BuildContext context, Route<T> route, String routeName, Widget widget}) {
    if(context == null) context = _context;
    if(route != null) return Navigator.push(context, route); else
    if(widget != null) return Navigator.push(context, MaterialPageRoute(builder: (context) => widget, settings: routeName != null ? RouteSettings(name: routeName) : null));
    return null;
  }

  @override
  Future<T> pushReplacement<T, TO>({BuildContext context, Route<T> newRoute, String newRouteName, Widget newWidget, TO result}) {
    if(context == null) context = _context;
    if(newRoute != null) return Navigator.pushReplacement(context, newRoute); else
    if(newWidget != null) return Navigator.pushReplacement(context, MaterialPageRoute(builder: (context) => newWidget, settings: newRouteName != null ? RouteSettings(name: newRouteName) : null));
    return null;
  }

  @override
  void onClick({Widget w, id}) {
    if(state != null) state.onClick(w: w, id: id);
  }

  @override
  void onLongClick({Widget w, id}) {
    if(state != null) state.onLongClick(w: w, id: id);
  }

  @override
  void onItemLongClick({Widget w, T item, int position, id}) {
    if(state != null) state.onItemLongClick(w: w, item: item, position: position, id: id);
  }

  @override
  void onItemClick({Widget w, T item, int position, id}) {
    if(state != null) state.onItemClick(w: w, item: item, position: position, id: id);
  }

  @override
  void onCreated() {

  }

  @override
  void onResumed() {

  }

  @override
  void onInactive() {

  }

  @override
  void onPaused() {

  }

  @override
  void onSuspending() {

  }

  @override
  void onStopped() {

  }

  @override
  void onDestroy() {

  }

}
